@@ -1,10 +1,12 @@
-import autopath
+from autopath import do_autopath
+do_autopath()
+
 from py.test import raises
 
-from tiramisu.setting import groups
 from tiramisu.config import Config
+from tiramisu.setting import groups, owners
 from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
-    StrOption, OptionDescription, SymLinkOption
+    StrOption, OptionDescription, SymLinkOption, IPOption, NetmaskOption
 from tiramisu.error import PropertiesOptionError, ConflictError, SlaveError, ConfigError
 
 
@@ -51,30 +53,6 @@ def is_config(config, **kwargs):
         return 'no'
 
 
-def make_description():
-    gcoption = ChoiceOption('name', 'GC name', ('ref', 'framework'), 'ref')
-    gcdummy = BoolOption('dummy', 'dummy', default=False)
-    objspaceoption = ChoiceOption('objspace', 'Object space',
-                                  ('std', 'thunk'), 'std')
-    booloption = BoolOption('bool', 'Test boolean option', default=True)
-    intoption = IntOption('int', 'Test int option', default=0)
-    intoption2 = IntOption('int', 'Test int option', default=0)
-    floatoption = FloatOption('float', 'Test float option', default=2.3)
-    stroption = StrOption('str', 'Test string option', default="abc")
-    boolop = BoolOption('boolop', 'Test boolean option op', default=True)
-    wantref_option = BoolOption('wantref', 'Test requires', default=False,
-                                requires=({'option': boolop, 'expected': True, 'action': 'hidden'},))
-    wantframework_option = BoolOption('wantframework', 'Test requires',
-                                      default=False,
-                                      requires=({'option': boolop, 'expected': True, 'action': 'hidden'},))
-    gcgroup = OptionDescription('gc', '', [gcoption, gcdummy, floatoption, intoption2])
-    descr = OptionDescription('constraints', '', [gcgroup, booloption, objspaceoption,
-                              wantref_option, stroption,
-                              wantframework_option,
-                              intoption, boolop])
-    return descr
-
-
 def make_description_duplicates():
     gcoption = ChoiceOption('name', 'GC name', ('ref', 'framework'), 'ref')
     ## dummy 1
@@ -258,6 +236,7 @@ def test_callback_invalid():
     raises(ValueError, 'val1 = StrOption("val1", "", callback="string")')
     raises(ValueError, 'val1 = StrOption("val1", "", callback=return_val, callback_params="string")')
     val1 = StrOption('val1', "", 'val')
+    val1
     raises(ValueError, "StrOption('val2', '', callback=return_value, callback_params={'': 'string'})")
     raises(ValueError, "StrOption('val4', '', callback=return_value, callback_params={'value': (('string', False),)})")
     raises(ValueError, "StrOption('val4', '', callback=return_value, callback_params={'value': ((val1, 'string'),)})")
@@ -348,6 +327,7 @@ def test_callback_symlink():
     cfg = Config(maconfig)
     cfg.read_write()
     assert cfg.val1 == 'val'
+    assert cfg.val2 == 'val'
     assert cfg.val3 == 'val'
     cfg.val1 = 'new-val'
     assert cfg.val1 == 'new-val'
@@ -430,6 +410,18 @@ def test_callback_multi_list_extend():
     assert cfg.val1 == ['1', '2', '3', '4', '5']
 
 
+def test_callback_multi_callback():
+    val1 = StrOption('val1', "", multi=True, callback=return_val)
+    interface1 = OptionDescription('val1', '', [val1])
+    maconfig = OptionDescription('rootconfig', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.val1.val1 == ['val']
+    cfg.val1.val1 = ['val1']
+    cfg.val1.val1.append()
+    assert cfg.val1.val1 == ['val1', 'val']
+
+
 def test_callback_master_and_slaves_master():
     val1 = StrOption('val1', "", multi=True, callback=return_val)
     val2 = StrOption('val2', "", multi=True)
@@ -460,6 +452,74 @@ def test_callback_master_and_slaves_master2():
     assert cfg.val1.val2 == ['val2']
 
 
+def test_callback_master_and_slaves_master3():
+    val1 = StrOption('val1', "", multi=True, properties=('mandatory', 'empty'))
+    val2 = StrOption('val2', "", multi=True, default_multi='val2', properties=('expert',))
+    val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ((val2, False),)})
+    val4 = StrOption('val4', "", multi=True, callback=return_value, callback_params={'': ((val3, False),)})
+    interface1 = OptionDescription('val1', '', [val1, val2, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings()) == ['val1.val1']
+
+
+def test_callback_master_and_slaves_master4():
+    val1 = StrOption('val1', "", ['val1'], multi=True, properties=('mandatory',))
+    val2 = StrOption('val2', "", multi=True, default_multi='val2', properties=('expert', 'mandatory'))
+    val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ((val2, False),)})
+    val4 = StrOption('val4', "", multi=True, callback=return_value, callback_params={'': ((val3, False),)})
+    interface1 = OptionDescription('val1', '', [val1, val2, val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.cfgimpl_get_settings().append('expert')
+    cfg.cfgimpl_get_settings().setpermissive(('expert',))
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == []
+
+
+def test_callback_master_and_slaves_master_mandatory_transitive():
+    #default value
+    val1 = IPOption('val1', "", ['192.168.0.1'], multi=True, properties=('mandatory',))
+    val2 = NetmaskOption('val2', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
+    val2.impl_add_consistency('ip_netmask', val1)
+    #no value
+    val3 = IPOption('val3', "", multi=True, properties=('mandatory',))
+    val4 = NetmaskOption('val4', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
+    val4.impl_add_consistency('ip_netmask', val3)
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface2 = OptionDescription('val3', '', [val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    interface2.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1, interface2])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    raises(PropertiesOptionError, "cfg.val1.val1")
+    raises(PropertiesOptionError, "cfg.val3.val3")
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == []
+
+
+def test_callback_master_and_slaves_master_mandatory_non_transitive():
+    #no value
+    val1 = IPOption('val1', "", multi=True, properties=('mandatory',))
+    val2 = NetmaskOption('val2', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
+    val2.impl_add_consistency('ip_netmask', val1, transitive=False)
+    #default value
+    val3 = IPOption('val3', "", ['192.168.0.1'], multi=True, properties=('mandatory',))
+    val4 = NetmaskOption('val4', "", multi=True, default_multi='255.255.255.0', properties=('disabled', 'mandatory'))
+    val4.impl_add_consistency('ip_netmask', val3, transitive=False)
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface2 = OptionDescription('val3', '', [val3, val4])
+    interface1.impl_set_group_type(groups.master)
+    interface2.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1, interface2])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert list(cfg.cfgimpl_get_values().mandatory_warnings(force_permissive=True)) == ["val1.val1"]
+
+
 def test_callback_master_and_slaves_master_list():
     val1 = StrOption('val1', "", multi=True, callback=return_list)
     val2 = StrOption('val2', "", multi=True)
@@ -541,8 +601,6 @@ def test_callback_master_and_slaves_slave_cal():
     cfg.val3 = ['val1']
     assert cfg.val1.val1 == ['val1']
     assert cfg.val1.val2 == ['val']
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == ['val']
     del(cfg.val1.val1)
     cfg.val1.val2 = ['val']
     cfg.val3 = ['val1', 'val2']
@@ -558,55 +616,91 @@ def test_callback_master_and_slaves_slave_cal():
     assert cfg.val1.val2 == ['val1', 'val2', 'val']
 
 
-def test_callback_master_and_slaves_slave_cal2():
-    val3 = StrOption('val3', "", ['val', 'val'], multi=True)
-    val1 = StrOption('val1', "", multi=True, callback=return_value, callback_params={'': ((val3, False),)})
-    val2 = StrOption('val2', "", ['val2', 'val2'], multi=True)
-    interface1 = OptionDescription('val1', '', [val1, val2])
-    interface1.impl_set_group_type(groups.master)
-    maconfig = OptionDescription('rootconfig', '', [interface1, val3])
-    cfg = Config(maconfig)
-    cfg.read_write()
-    assert cfg.val3 == ['val', 'val']
-    assert cfg.val1.val1 == ['val', 'val']
-    assert cfg.val1.val2 == ['val2', 'val2']
-    cfg.val3.pop(1)
-#    # cannot remove slave's value because master is calculated
-#    # so raise
-    raises(SlaveError, "cfg.val1.val1")
-    raises(SlaveError, "cfg.val1.val2")
-    cfg.val3 = ['val', 'val']
-    assert cfg.val3 == ['val', 'val']
-    assert cfg.val1.val1 == ['val', 'val']
-    assert cfg.val1.val2 == ['val2', 'val2']
-    raises(SlaveError, "cfg.val1.val1 = ['val']")
-    assert cfg.val3 == ['val', 'val']
-    assert cfg.val1.val1 == ['val', 'val']
-    assert cfg.val1.val2 == ['val2', 'val2']
-
-
-def test_callback_master_and_slaves_slave_list():
-    val1 = StrOption('val1', "", multi=True)
-    val2 = StrOption('val2', "", multi=True, callback=return_list)
+def test_callback_master_and_slaves_master_disabled():
+    #properties must be transitive
+    val1 = StrOption('val1', "", multi=True, properties=('disabled',))
+    val2 = StrOption('val2', "", multi=True)
     interface1 = OptionDescription('val1', '', [val1, val2])
     interface1.impl_set_group_type(groups.master)
     maconfig = OptionDescription('rootconfig', '', [interface1])
     cfg = Config(maconfig)
     cfg.read_write()
+    raises(PropertiesOptionError, "cfg.val1.val1")
+    raises(PropertiesOptionError, "cfg.val1.val1.append('yes')")
+    raises(PropertiesOptionError, "cfg.val1.val2")
+
+
+def test_callback_master_and_slaves_master_callback_disabled():
+    val0 = StrOption('val0', "", multi=True, properties=('disabled',))
+    val1 = StrOption('val1', "", multi=True, callback=return_value, callback_params={'': ((val0, False),)})
+    val2 = StrOption('val2', "", multi=True)
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1, val0])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    raises(ConfigError, "cfg.val1.val1")
+    raises(ConfigError, "cfg.val1.val2")
+    cfg.cfgimpl_get_settings().remove('disabled')
+    cfg.val1.val1 = []
+    cfg.cfgimpl_get_settings().append('disabled')
+    assert cfg.val1.val1 == []
     assert cfg.val1.val2 == []
-    cfg.val1.val1 = ['val1', 'val2']
-    assert cfg.val1.val1 == ['val1', 'val2']
-    assert cfg.val1.val2 == ['val', 'val']
-    cfg.val1.val1 = ['val1']
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == ['val']
+
+
+def test_callback_master_and_slaves_slave_disabled():
+    val1 = StrOption('val1', "", multi=True)
+    val2 = StrOption('val2', "", multi=True, properties=('disabled',))
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.val1.val1 == []
+    raises(PropertiesOptionError, "cfg.val1.val2")
+    cfg.val1.val1.append('yes')
+    assert cfg.val1.val1 == ['yes']
+    cfg.cfgimpl_get_settings().remove('disabled')
+    assert cfg.val1.val2 == [None]
+    cfg.val1.val2 = ['no']
+    cfg.val1.val1.append('yes2')
+    cfg.val1.val1.append('yes3')
+    cfg.val1.val2[2] = 'no1'
+    assert cfg.val1.val2 == ['no', None, 'no1']
+    cfg.cfgimpl_get_settings().append('disabled')
+    cfg.val1.val1.pop(0)
+    assert cfg.val1.val1 == ['yes2', 'yes3']
+    cfg.cfgimpl_get_settings().remove('disabled')
+    assert cfg.val1.val2 == [None, 'no1']
+
+
+def test_callback_master_and_slaves_slave_callback_disabled():
+    val0 = StrOption('val0', "", multi=True, properties=('disabled',))
+    val1 = StrOption('val1', "", multi=True)
+    val2 = StrOption('val2', "", multi=True, callback=return_value, callback_params={'': ((val0, False),)})
+    interface1 = OptionDescription('val1', '', [val1, val2])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('rootconfig', '', [interface1, val0])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.val1.val1 == []
+    assert cfg.val1.val2 == []
+    cfg.val1.val1.append('yes')
+    assert cfg.val1.val1 == ['yes']
+    cfg.cfgimpl_get_settings().remove('disabled')
+    raises(SlaveError, "cfg.val1.val2")
+    cfg.val1.val2 = ['no']
+    cfg.val1.val1.append('yes1')
+    raises(SlaveError, "cfg.val1.val2")
+    cfg.cfgimpl_get_settings().append('disabled')
+    raises(ConfigError, "cfg.val1.val1.pop(0)")
 
 
 def test_callback_master_and_slaves_value():
+    val4 = StrOption('val4', '', multi=True, default=['val10', 'val11'])
     val1 = StrOption('val1', "", multi=True)
     val2 = StrOption('val2', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)})
     val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ('yes',)})
-    val4 = StrOption('val4', '', multi=True, default=['val10', 'val11'])
     val5 = StrOption('val5', "", multi=True, callback=return_value, callback_params={'': ((val4, False),)})
     val6 = StrOption('val6', "", multi=True, callback=return_value, callback_params={'': ((val5, False),)})
     interface1 = OptionDescription('val1', '', [val1, val2, val3, val5, val6])
@@ -614,39 +708,41 @@ def test_callback_master_and_slaves_value():
     maconfig = OptionDescription('rootconfig', '', [interface1, val4])
     cfg = Config(maconfig)
     cfg.read_write()
+    cfg.val4 == ['val10', 'val11']
     assert cfg.val1.val1 == []
     assert cfg.val1.val2 == []
     assert cfg.val1.val3 == []
     assert cfg.val1.val5 == []
     assert cfg.val1.val6 == []
+    #raises(SlaveError, "cfg.val1.val1")
+    #raises(SlaveError, "cfg.val1.val2")
+    #raises(SlaveError, "cfg.val1.val3")
+    #raises(SlaveError, "cfg.val1.val5")
+    #raises(SlaveError, "cfg.val1.val6")
     #
-    cfg.val1.val1 = ['val1']
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == ['val1']
-    assert cfg.val1.val3 == ['yes']
-    assert cfg.val1.val5 == ['val10']
-    assert cfg.val1.val6 == ['val10']
+    #default calculation has greater length
+    #raises(SlaveError, "cfg.val1.val1 = ['val1']")
     #
-    cfg.val1.val1.append('val2')
+    cfg.val1.val1 = ['val1', 'val2']
     assert cfg.val1.val1 == ['val1', 'val2']
     assert cfg.val1.val2 == ['val1', 'val2']
     assert cfg.val1.val3 == ['yes', 'yes']
-    assert cfg.val1.val5 == ['val10', 'val11']
-    assert cfg.val1.val6 == ['val10', 'val11']
+    raises(SlaveError, "cfg.val1.val5")
+    raises(SlaveError, "cfg.val1.val6")
     #
-    cfg.val1.val1 = ['val1', 'val2', 'val3']
+    cfg.val1.val1.append('val3')
     assert cfg.val1.val1 == ['val1', 'val2', 'val3']
     assert cfg.val1.val2 == ['val1', 'val2', 'val3']
     assert cfg.val1.val3 == ['yes', 'yes', 'yes']
-    assert cfg.val1.val5 == ['val10', 'val11', None]
-    assert cfg.val1.val6 == ['val10', 'val11', None]
+    raises(SlaveError, "cfg.val1.val5")
+    raises(SlaveError, "cfg.val1.val6")
     #
     cfg.val1.val1.pop(2)
     assert cfg.val1.val1 == ['val1', 'val2']
     assert cfg.val1.val2 == ['val1', 'val2']
     assert cfg.val1.val3 == ['yes', 'yes']
-    assert cfg.val1.val5 == ['val10', 'val11']
-    assert cfg.val1.val6 == ['val10', 'val11']
+    raises(SlaveError, "cfg.val1.val5")
+    raises(SlaveError, "cfg.val1.val6")
     #
     cfg.val1.val2 = ['val2', 'val2']
     cfg.val1.val3 = ['val2', 'val2']
@@ -659,158 +755,27 @@ def test_callback_master_and_slaves_value():
     cfg.val1.val1.append('val3')
     assert cfg.val1.val2 == ['val2', 'val2', 'val3']
     assert cfg.val1.val3 == ['val2', 'val2', 'yes']
-    assert cfg.val1.val5 == ['val2', 'val2', None]
-    assert cfg.val1.val6 == ['val2', 'val2', None]
+    raises(SlaveError, "cfg.val1.val5")
+    raises(SlaveError, "cfg.val1.val6")
     cfg.cfgimpl_get_settings().remove('cache')
     cfg.val4 = ['val10', 'val11', 'val12']
     #if value is already set, not updated !
     cfg.val1.val1.pop(2)
     cfg.val1.val1.append('val3')
     cfg.val1.val1 = ['val1', 'val2', 'val3']
-    assert cfg.val1.val5 == ['val2', 'val2', 'val12']
-    assert cfg.val1.val6 == ['val2', 'val2', 'val12']
-
-
-def test_callback_master_and_slaves_disabled():
-    val = BoolOption('val', '', multi=True)
-    val1 = StrOption('val1', "", multi=True)
-    val2 = StrOption('val2', "", multi=True, callback=return_value, callback_params={'': ((val, False),)})
-    interface1 = OptionDescription('val1', '', [val1, val2])
-    interface1.impl_set_group_type(groups.master)
-    maconfig = OptionDescription('rootconfig', '', [val, interface1])
-    cfg = Config(maconfig)
-    cfg.read_write()
-    #
-    assert cfg.val == []
-    assert cfg.val1.val1 == []
-    assert cfg.val1.val2 == []
-    cfg.val1.val1.append('val1')
-    assert cfg.val == []
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == [None]
-    #
-    cfg.val.append(True)
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val1.val2].append('disabled')
-    assert cfg.val == [True]
-    assert cfg.val1.val1 == ['val1']
-    raises(PropertiesOptionError, 'cfg.val1.val2')
-    #
-    cfg.val1.val1.append('val1_1')
-    assert cfg.val == [True]
-    assert cfg.val1.val1 == ['val1', 'val1_1']
-    raises(PropertiesOptionError, 'cfg.val1.val2')
-    #
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val1.val2].remove('disabled')
-    assert cfg.val == [True]
-    assert cfg.val1.val1 == ['val1', 'val1_1']
-    raises(ValueError, 'cfg.val1.val2')
-    #
-    cfg.val = []
-    assert cfg.val1.val1 == ['val1', 'val1_1']
-    assert cfg.val1.val2 == [None, None]
-    #
-    cfg.val1.val2 = [None, None]
-    cfg.val = [True, True]
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val1.val2].append('disabled')
-    cfg.val1.val1.pop(1)
-    assert cfg.val == [True, True]
-    assert cfg.val1.val1 == ['val1']
-    raises(PropertiesOptionError, 'cfg.val1.val2')
-    #
-    cfg.val1.val1.append('val1_2')
-    assert cfg.val == [True, True]
-    assert cfg.val1.val1 == ['val1', 'val1_2']
-    raises(PropertiesOptionError, 'cfg.val1.val2')
-    #
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val1.val2].remove('disabled')
-    assert cfg.val == [True, True]
-    assert cfg.val1.val1 == ['val1', 'val1_2']
-    #[None, None] + pop() + append() => [None, True]
-    raises(ValueError, 'assert cfg.val1.val2')
-
-
-def test_callback_master_and_slaves_requires():
-    val = StrOption('val', '', 'val')
-    valreq = StrOption('valreq', '', 'val')
-    val1 = StrOption('val1', "", multi=True)
-    val2 = StrOption('val2', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)})
-    val3 = StrOption('val3', "", multi=True, callback=return_value, callback_params={'': ('yes',)})
-    val4 = StrOption('val4', '', multi=True, default=['val10', 'val11'])
-    val5 = StrOption('val5', "", multi=True, callback=return_value, callback_params={'': ((val4, False),)})
-    val6 = StrOption('val6', "", multi=True, callback=return_value, callback_params={'': ((val, False),)},
-                     requires=({'option': valreq, 'expected': 'val_disabled', 'action': 'disabled'},))
-    interface1 = OptionDescription('val1', '', [val1, val2, val3, val5, val6])
-    interface1.impl_set_group_type(groups.master)
-    maconfig = OptionDescription('rootconfig', '', [val, valreq, interface1, val4])
-    cfg = Config(maconfig)
-    cfg.read_write()
-    assert cfg.val1.val1 == []
-    assert cfg.val1.val2 == []
-    assert cfg.val1.val3 == []
-    assert cfg.val1.val5 == []
-    assert cfg.val1.val6 == []
-    #
-    cfg.val1.val1 = ['val1']
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == ['val1']
-    assert cfg.val1.val3 == ['yes']
-    assert cfg.val1.val5 == ['val10']
-    assert cfg.val1.val6 == ['val']
-    #
-    cfg.valreq = 'val_disabled'
-    assert cfg.val1.val1 == ['val1']
-    assert cfg.val1.val2 == ['val1']
-    assert cfg.val1.val3 == ['yes']
-    assert cfg.val1.val5 == ['val10']
-    raises(PropertiesOptionError, 'cfg.val1.val6')
-    assert cfg.make_dict() == {'val1.val2': ['val1'], 'val1.val3': ['yes'], 'val1.val1': ['val1'], 'val1.val5': ['val10'], 'val': 'val', 'valreq': 'val_disabled', 'val4': ['val10', 'val11']}
-    #
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val].append('disabled')
-    cfg.cfgimpl_get_settings()[cfg.cfgimpl_get_description().val1.val3].append('disabled')
-    raises(PropertiesOptionError, 'cfg.val1.val6')
-    assert cfg.make_dict() == {'val1.val2': ['val1'], 'val1.val1': ['val1'], 'val1.val5': ['val10'], 'val4': ['val10', 'val11'], 'valreq': 'val_disabled'}
-    #
-    cfg.valreq = 'val'
-    raises(ConfigError, 'cfg.val1.val6')
+    raises(SlaveError, "cfg.val1.val5")
+    raises(SlaveError, "cfg.val1.val6")
 
 
 def test_callback_master():
     val2 = StrOption('val2', "", multi=True, callback=return_value)
     val1 = StrOption('val1', "", multi=True, callback=return_value, callback_params={'': ((val2, False),)})
     interface1 = OptionDescription('val1', '', [val1, val2])
+    interface1
     raises(ValueError, "interface1.impl_set_group_type(groups.master)")
 
 
-def test_callback_master_and_other_master_slave():
-    val1 = StrOption('val1', "", multi=True)
-    val2 = StrOption('val2', "", multi=True)
-    val3 = StrOption('val3', "", multi=True)
-    val4 = StrOption('val4', '', multi=True, default=['val10', 'val11'])
-    val5 = StrOption('val5', "", multi=True, callback=return_value, callback_params={'': ((val1, False),)})
-    val6 = StrOption('val6', "", multi=True, callback=return_value, callback_params={'': ((val2, False),)})
-    interface1 = OptionDescription('val1', '', [val1, val2, val3])
-    interface1.impl_set_group_type(groups.master)
-    interface2 = OptionDescription('val4', '', [val4, val5, val6])
-    interface2.impl_set_group_type(groups.master)
-    maconfig = OptionDescription('rootconfig', '', [interface1, interface2])
-    cfg = Config(maconfig)
-    cfg.read_write()
-    assert cfg.val4.val4 == ['val10', 'val11']
-    assert cfg.val4.val5 == [None, None]
-    assert cfg.val4.val6 == [None, None]
-    cfg.val1.val1 = ['yes']
-    assert cfg.val4.val4 == ['val10', 'val11']
-    assert cfg.val4.val5 == ['yes', None]
-    assert cfg.val4.val6 == [None, None]
-    cfg.val1.val2 = ['no']
-    assert cfg.val4.val4 == ['val10', 'val11']
-    assert cfg.val4.val5 == ['yes', None]
-    assert cfg.val4.val6 == ['no', None]
-    cfg.val1.val1 = ['yes', 'yes', 'yes']
-    cfg.val1.val2 = ['no', 'no', 'no']
-    assert cfg.val4.val4 == ['val10', 'val11']
-    assert cfg.val4.val5 == ['yes', 'yes']
-    assert cfg.val4.val6 == ['no', 'no']
+#FIXME: slave est un symlink
 
 
 def test_callback_different_type():
@@ -860,6 +825,19 @@ def test_callback_two_disabled():
     raises(PropertiesOptionError, 'cfg.od2.opt2')
 
 
+def test_callback_two_disabled2():
+    opt1 = BoolOption('opt1', '', properties=('hidden',))
+    opt2 = BoolOption('opt2', '', callback=return_value, callback_params={'': ((opt1, False),)}, properties=('hidden',))
+    od1 = OptionDescription('od1', '', [opt1])
+    od2 = OptionDescription('od2', '', [opt2])
+    maconfig = OptionDescription('rootconfig', '', [od1, od2])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.cfgimpl_get_settings().setpermissive(('hidden',))
+    raises(PropertiesOptionError, 'cfg.od2.opt2')
+    assert cfg.getowner(opt2, force_permissive=True) == owners.default
+
+
 def test_callback_calculating_invalid():
     opt1 = IntOption('opt1', '', 1)
     opt2 = BoolOption('opt2', '', callback=return_value, callback_params={'': ((opt1, False),)})
@@ -896,6 +874,17 @@ def test_callback_calculating_mandatory():
     raises(ConfigError, 'cfg.od2.opt2')
 
 
+def test_callback_calculating_mandatory_multi():
+    opt1 = BoolOption('opt1', '', multi=True, properties=('disabled',))
+    opt2 = BoolOption('opt2', '', multi=True, callback=return_value, callback_params={'': ((opt1, False),)}, properties=('mandatory',))
+    od1 = OptionDescription('od1', '', [opt1])
+    od2 = OptionDescription('od2', '', [opt2])
+    maconfig = OptionDescription('rootconfig', '', [od1, od2])
+    cfg = Config(maconfig)
+    cfg.read_only()
+    raises(ConfigError, 'cfg.od2.opt2')
+
+
 def test_callback_two_disabled_multi():
     opt1 = BoolOption('opt1', '', properties=('disabled',))
     opt2 = BoolOption('opt2', '', callback=return_value, callback_params={'': ((opt1, False),)}, properties=('disabled',), multi=True)
@@ -925,3 +914,33 @@ def test_callback_multi_list_params_key():
     cfg = Config(maconfig)
     cfg.read_write()
     assert cfg.val2.val2 == ['val', 'val']
+
+
+def test_masterslaves_callback_description():
+    st1 = StrOption('st1', "", multi=True)
+    st2 = StrOption('st2', "", multi=True, callback=return_value, callback_params={'': ((st1, False),)})
+    stm = OptionDescription('st1', '', [st1, st2])
+    stm.impl_set_group_type(groups.master)
+    st = OptionDescription('st', '', [stm])
+    od = OptionDescription('od', '', [st])
+    od2 = OptionDescription('od', '', [od])
+    cfg = Config(od2)
+    owner = cfg.cfgimpl_get_settings().getowner()
+    st1 = cfg.unwrap_from_path('od.st.st1.st1')
+    st2 = cfg.unwrap_from_path('od.st.st1.st2')
+    assert cfg.od.st.st1.st1 == []
+    assert cfg.od.st.st1.st2 == []
+    assert cfg.getowner(st1) == owners.default
+    assert cfg.getowner(st2) == owners.default
+    ##
+    cfg.od.st.st1.st1.append('yes')
+    assert cfg.od.st.st1.st1 == ['yes']
+    assert cfg.od.st.st1.st2 == ['yes']
+    assert cfg.getowner(st1) == owner
+
+
+def test_re_set_callback():
+    st1 = StrOption('st1', "", multi=True)
+    st2 = StrOption('st2', "", multi=True)
+    st2.impl_set_callback(return_value, {'': ((st1, False),)})
+    raises(ConfigError, "st2.impl_set_callback(return_value, {'': ((st1, False),)})")
