@@ -1,8 +1,10 @@
 "config.set() or config.setoption() or option.setoption()"
-import autopath
+from autopath import do_autopath
+do_autopath()
+
 from py.test import raises
 
-from tiramisu.setting import owners
+from tiramisu.setting import owners, groups
 from tiramisu.config import Config
 from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
     StrOption, OptionDescription
@@ -94,6 +96,7 @@ def test_default_with_multi():
 def test_idontexist():
     descr = make_description()
     cfg = Config(descr)
+    cfg
     raises(AttributeError, "cfg.idontexist")
 
 
@@ -158,7 +161,6 @@ def test__requires_with_inverted():
 def test_multi_with_requires_in_another_group():
     s = StrOption("string", "", default=["string"], multi=True)
     intoption = IntOption('int', 'Test int option', default=0)
-    descr = OptionDescription("options", "", [intoption])
     stroption = StrOption('str', 'Test string option', default=["abc"],
                           requires=[{'option': intoption, 'expected': 1, 'action': 'hidden'}], multi=True)
     descr = OptionDescription("opt", "", [stroption])
@@ -175,7 +177,6 @@ def test_multi_with_requires_in_another_group():
 def test_apply_requires_from_config():
     s = StrOption("string", "", default=["string"], multi=True)
     intoption = IntOption('int', 'Test int option', default=0)
-    descr = OptionDescription("options", "", [intoption])
     stroption = StrOption('str', 'Test string option', default=["abc"],
                           requires=[{'option': intoption, 'expected': 1, 'action': 'hidden'}], multi=True)
     descr = OptionDescription("opt", "", [stroption])
@@ -192,7 +193,6 @@ def test_apply_requires_from_config():
 def test_apply_requires_with_disabled():
     s = StrOption("string", "", default=["string"], multi=True)
     intoption = IntOption('int', 'Test int option', default=0)
-    descr = OptionDescription("options", "", [intoption])
     stroption = StrOption('str', 'Test string option', default=["abc"],
                           requires=[{'option': intoption, 'expected': 1, 'action': 'disabled'}], multi=True)
     descr = OptionDescription("opt", "", [stroption])
@@ -209,7 +209,6 @@ def test_apply_requires_with_disabled():
 def test_multi_with_requires_with_disabled_in_another_group():
     s = StrOption("string", "", default=["string"], multi=True)
     intoption = IntOption('int', 'Test int option', default=0)
-    descr = OptionDescription("options", "", [intoption])
     stroption = StrOption('str', 'Test string option', default=["abc"],
                           requires=[{'option': intoption, 'expected': 1, 'action': 'disabled'}], multi=True)
     descr = OptionDescription("opt", "", [stroption])
@@ -224,9 +223,50 @@ def test_multi_with_requires_with_disabled_in_another_group():
 
 
 def test_multi_with_requires_that_is_multi():
-    s = StrOption("string", "", default=["string"], multi=True)
-    intoption = IntOption('int', 'Test int option', default=[0], multi=True)
-    raises(ValueError, "StrOption('str', 'Test string option', default=['abc'], requires=[{'option': intoption, 'expected': 1, 'action': 'hidden'}], multi=True)")
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', default=['abc'], requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
+    descr = OptionDescription("opt", "", [b, c])
+    descr
+    raises(ValueError, "Config(descr)")
+
+
+def test_multi_with_requires_that_is_masterslave():
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
+    descr = OptionDescription("int", "", [b, c])
+    descr.impl_set_group_type(groups.master)
+    Config(descr)
+
+
+def test_multi_with_requires_that_is_masterslave_master():
+    b = IntOption('int', 'Test int option', multi=True)
+    c = StrOption('str', 'Test string option', requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
+    descr = OptionDescription("str", "", [c, b])
+    descr.impl_set_group_type(groups.master)
+    raises(ValueError, "Config(descr)")
+
+
+def test_multi_with_requires_that_is_masterslave2():
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', multi=True)
+    d = StrOption('str1', 'Test string option', requires=[{'option': c, 'expected': '1', 'action': 'hidden'}], multi=True)
+    descr = OptionDescription("int", "", [b, c, d])
+    descr.impl_set_group_type(groups.master)
+    Config(descr)
+
+
+def test_multi_with_requires_that_is_not_same_masterslave():
+    b = IntOption('int', 'Test int option', default=[0], multi=True)
+    c = StrOption('str', 'Test string option', requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
+    descr1 = OptionDescription("int", "", [b, c])
+    descr1.impl_set_group_type(groups.master)
+    d = IntOption('int1', 'Test int option', default=[0], multi=True)
+    e = StrOption('str', 'Test string option', requires=[{'option': b, 'expected': 1, 'action': 'hidden'}], multi=True)
+    descr2 = OptionDescription("int1", "", [d, e])
+    descr2.impl_set_group_type(groups.master)
+    descr3 = OptionDescription('val', '', [descr1, descr2])
+    descr3
+    raises(ValueError, "Config(descr3)")
 
 
 def test_multi_with_bool():
@@ -243,6 +283,7 @@ def test_multi_with_bool_two():
     s = BoolOption("bool", "", default=[False], multi=True)
     descr = OptionDescription("options", "", [s])
     config = Config(descr)
+    config
     assert descr.bool.impl_is_multi() is True
     raises(ValueError, "config.bool = True")
 
@@ -314,7 +355,7 @@ def test_extend_config_properties():
     descr = make_description()
     cfg = Config(descr)
     setting = cfg.cfgimpl_get_settings()
-    assert set(eval(str(setting))) == set(['cache', 'validator', 'expire', 'warnings'])
+    assert set(eval(str(setting))) == set(['cache', 'expire', 'validator', 'warnings'])
     setting.extend(['test', 'test2'])
     assert set(eval(str(setting))) == set(['test', 'cache', 'test2', 'expire', 'validator', 'warnings'])
 
@@ -324,10 +365,10 @@ def test_append_properties():
     cfg = Config(descr)
     setting = cfg.cfgimpl_get_settings()
     option = cfg.cfgimpl_get_description().gc.dummy
-    assert option._properties == tuple()
+    assert tuple(option._properties) == tuple()
     assert not 'test' in setting[option]
     setting[option].append('test')
-    assert option._properties == tuple()
+    assert tuple(option._properties) == tuple()
     assert 'test' in setting[option]
 
 
@@ -385,5 +426,48 @@ def test_properties_cached():
     c.read_write()
     setting = c.cfgimpl_get_settings()
     option = c.cfgimpl_get_description().sub.b1
+    option
     c._setattr('sub.b1', True, force_permissive=True)
-    assert str(setting[b1]) == "['test']"
+    assert str(setting[b1]) in ["['test']", "[u'test']"]
+
+
+
+def test_append_properties_force_store_value():
+    gcdummy = BoolOption('dummy', 'dummy', default=False, properties=('force_store_value',))
+    gcgroup = OptionDescription('gc', '', [gcdummy])
+    descr = OptionDescription('tiramisu', '', [gcgroup])
+    cfg = Config(descr)
+    setting = cfg.cfgimpl_get_settings()
+    option = cfg.cfgimpl_get_description().gc.dummy
+    assert tuple(option._properties) == tuple(['force_store_value'])
+    assert not 'test' in setting[option]
+    setting[option].append('test')
+    assert tuple(option._properties) == tuple(['force_store_value'])
+    assert 'test' in setting[option]
+
+
+def test_reset_properties_force_store_value():
+    gcdummy = BoolOption('dummy', 'dummy', default=False, properties=('force_store_value',))
+    gcgroup = OptionDescription('gc', '', [gcdummy])
+    descr = OptionDescription('tiramisu', '', [gcgroup])
+    cfg = Config(descr)
+    setting = cfg.cfgimpl_get_settings()
+    option = cfg.cfgimpl_get_description().gc.dummy
+    assert setting._p_.get_modified_properties() == {}
+    setting.append('frozen')
+    assert setting._p_.get_modified_properties() == {None: set(('frozen', 'expire', 'cache', 'validator', 'warnings'))}
+    setting.reset()
+    assert setting._p_.get_modified_properties() == {}
+    setting[option].append('test')
+    assert setting._p_.get_modified_properties() == {'gc.dummy': set(('test', 'force_store_value'))}
+    setting.reset()
+    assert setting._p_.get_modified_properties() == {'gc.dummy': set(('test', 'force_store_value'))}
+    setting.append('frozen')
+    assert setting._p_.get_modified_properties() == {None: set(('frozen', 'expire', 'validator', 'cache', 'warnings')), 'gc.dummy': set(('test', 'force_store_value'))}
+    setting.reset(option)
+    assert setting._p_.get_modified_properties() == {None: set(('frozen', 'expire', 'validator', 'cache', 'warnings'))}
+    setting[option].append('test')
+    assert setting._p_.get_modified_properties() == {None: set(('frozen', 'expire', 'validator', 'cache', 'warnings')), 'gc.dummy': set(('test', 'force_store_value'))}
+    setting.reset(all_properties=True)
+    assert setting._p_.get_modified_properties() == {}
+    raises(ValueError, 'setting.reset(all_properties=True, opt=option)')
