@@ -1,5 +1,7 @@
 "configuration objects global API"
-import autopath
+from autopath import do_autopath
+do_autopath()
+
 from py.test import raises
 
 from tiramisu.config import Config
