@@ -1,5 +1,7 @@
 # coding: utf-8
-import autopath
+from autopath import do_autopath
+do_autopath()
+
 from tiramisu.setting import groups, owners
 from tiramisu.config import Config
 from tiramisu.option import ChoiceOption, BoolOption, IntOption, \
@@ -20,6 +22,7 @@ def make_description():
                                       default=False)
     mode_conteneur_actif2 = BoolOption('mode_conteneur_actif2', "le serveur est en mode conteneur2",
                                        default=False, properties=('hidden',))
+
     adresse_serveur_ntp = StrOption('serveur_ntp', "adresse serveur ntp", multi=True)
     time_zone = ChoiceOption('time_zone', 'fuseau horaire du serveur',
                              ('Paris', 'Londres'), 'Paris')
@@ -239,13 +242,32 @@ def test_allowed_groups():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
     interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1
     raises(ValueError, "interface1.impl_set_group_type('toto')")
 
 
+def test_values_with_master_disabled_master():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
+    cfg.ip_admin_eth0.ip_admin_eth0.pop(0)
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
+    cfg.ip_admin_eth0.netmask_admin_eth0 = ["192.168.230.145"]
+    del(cfg.ip_admin_eth0.netmask_admin_eth0)
+    cfg.cfgimpl_get_settings()[ip_admin_eth0].append('disabled')
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0] = '192.168.230.145'")
+
+
 def test_master_not_valid_name():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
     invalid_group = OptionDescription('interface1', '', [ip_admin_eth0, netmask_admin_eth0])
+    invalid_group
     raises(ValueError, "invalid_group.impl_set_group_type(groups.master)")
 
 
@@ -254,6 +276,7 @@ def test_sub_group_in_master_group():
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
     subgroup = OptionDescription("subgroup", '', [])
     invalid_group = OptionDescription('ip_admin_eth0', '', [subgroup, ip_admin_eth0, netmask_admin_eth0])
+    invalid_group
     raises(ValueError, "invalid_group.impl_set_group_type(groups.master)")
 
 
@@ -261,6 +284,7 @@ def test_group_always_has_multis():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau")
     group = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    group
     raises(ValueError, "group.impl_set_group_type(groups.master)")
 
 
@@ -308,14 +332,15 @@ def test_reset_values_with_master_and_slaves():
     assert cfg.getowner(netmask_admin_eth0) == owners.default
     assert cfg.ip_admin_eth0.ip_admin_eth0 == []
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
-
+    #reset
     cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
-    cfg.cfgimpl_get_values().reset(ip_admin_eth0, force_permissive=True)
+    cfg.cfgimpl_get_values().reset(ip_admin_eth0)
     assert cfg.getowner(ip_admin_eth0) == owners.default
     assert cfg.getowner(netmask_admin_eth0) == owners.default
     assert cfg.ip_admin_eth0.ip_admin_eth0 == []
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
 
+
 def test_reset_values_with_master_and_slaves_default():
     ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True, default=['192.168.230.145'])
     netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
@@ -340,21 +365,50 @@ def test_reset_values_with_master_and_slaves_default():
     cfg.ip_admin_eth0.ip_admin_eth0[0] = "192.168.230.146"
     cfg.ip_admin_eth0.netmask_admin_eth0[0] = "255.255.255.0"
     assert cfg.getowner(ip_admin_eth0) == owner
-    assert cfg.getowner(netmask_admin_eth0) == owner
+    assert cfg.getowner(netmask_admin_eth0, 0) == owner
+    del(cfg.ip_admin_eth0.ip_admin_eth0)
+    assert cfg.getowner(ip_admin_eth0) == owners.default
+    assert cfg.getowner(netmask_admin_eth0, 0) == owners.default
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.230.145']
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None]
+
+    cfg.ip_admin_eth0.netmask_admin_eth0[0] = "255.255.255.0"
+    assert cfg.getowner(ip_admin_eth0) == owners.default
+    assert cfg.getowner(netmask_admin_eth0, 0) == owner
     del(cfg.ip_admin_eth0.ip_admin_eth0)
     assert cfg.getowner(ip_admin_eth0) == owners.default
     assert cfg.getowner(netmask_admin_eth0) == owners.default
     assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.230.145']
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None]
 
-    cfg.ip_admin_eth0.netmask_admin_eth0[0] = "255.255.255.0"
-    assert cfg.getowner(ip_admin_eth0) == owners.default
-    assert cfg.getowner(netmask_admin_eth0) == owner
-    del(cfg.ip_admin_eth0.ip_admin_eth0)
+
+def test_reset_values_with_master_and_slaves_setitem():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True, default=['192.168.230.145'])
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    owner = cfg.cfgimpl_get_settings().getowner()
     assert cfg.getowner(ip_admin_eth0) == owners.default
     assert cfg.getowner(netmask_admin_eth0) == owners.default
-    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.230.145']
-    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None]
+
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.146")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.146")
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.146")
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[-1] == None
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[3] == None
+    assert cfg.getowner(netmask_admin_eth0, 3) == owners.default
+    cfg.ip_admin_eth0.netmask_admin_eth0[-1] = "255.255.255.0"
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[-1] == "255.255.255.0"
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[3] == "255.255.255.0"
+    assert cfg.getowner(netmask_admin_eth0, 3) == owner
+    #
+    assert cfg.getowner(netmask_admin_eth0, 2) == owners.default
+    cfg.ip_admin_eth0.netmask_admin_eth0[2] = "255.255.0.0"
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[-2] == "255.255.0.0"
+    assert cfg.getowner(netmask_admin_eth0, 2) == owner
 
 
 def test_values_with_master_and_slaves_slave():
@@ -378,6 +432,11 @@ def test_values_with_master_and_slaves_slave():
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == ['255.255.255.0', None]
     cfg.ip_admin_eth0.netmask_admin_eth0 = ['255.255.255.0', '255.255.255.0']
     raises(SlaveError, 'cfg.ip_admin_eth0.netmask_admin_eth0.pop(1)')
+    #reset
+    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
+    cfg.cfgimpl_get_values().reset(ip_admin_eth0)
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == []
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
 
 
 def test_values_with_master_and_slaves_master():
@@ -400,11 +459,6 @@ def test_values_with_master_and_slaves_master():
     del(cfg.ip_admin_eth0.ip_admin_eth0)
     assert cfg.ip_admin_eth0.ip_admin_eth0 == []
     assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
-    #reset
-    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
-    cfg.cfgimpl_get_values().reset(ip_admin_eth0, force_permissive=True)
-    assert cfg.ip_admin_eth0.ip_admin_eth0 == []
-    assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
 
 
 def test_values_with_master_and_slaves_master_error():
@@ -475,23 +529,6 @@ def test_values_with_master_disabled():
     cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.230.43')
 
 
-def test_values_with_master_disabled_master():
-    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
-    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True)
-    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
-    interface1.impl_set_group_type(groups.master)
-    maconfig = OptionDescription('toto', '', [interface1])
-    cfg = Config(maconfig)
-    cfg.read_write()
-    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
-    cfg.ip_admin_eth0.ip_admin_eth0.pop(0)
-    cfg.ip_admin_eth0.ip_admin_eth0.append("192.168.230.145")
-    cfg.ip_admin_eth0.netmask_admin_eth0 = ["192.168.230.145"]
-    del(cfg.ip_admin_eth0.netmask_admin_eth0)
-    cfg.cfgimpl_get_settings()[ip_admin_eth0].append('disabled')
-    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0] = '192.168.230.145'")
-
-
 def test_multi_insert():
     var = StrOption('var', '', ['ok'], multi=True)
     od = OptionDescription('od', '', [var])
@@ -502,6 +539,7 @@ def test_multi_insert():
     c.var.insert(0, 'nok')
     assert c.var == ['nok', 'ok']
     assert c.getowner(var) != owners.default
+    raises(ValueError, 'c.var.insert(0, 1)')
 
 
 def test_multi_insert_master():
@@ -574,6 +612,7 @@ def test_multi_extend():
     c.var.extend(['pok'])
     assert c.var == ['ok', 'nok', 'pok']
     assert c.getowner(var) != owners.default
+    raises(ValueError, 'c.var.extend([1])')
 
 
 def test_multi_extend_master():
