@@ -1,6 +1,9 @@
 # coding: utf-8
-import autopath
+from autopath import do_autopath
+do_autopath()
+
 from copy import copy
+from tiramisu.setting import groups
 from tiramisu import setting
 setting.expires_time = 1
 from tiramisu.option import IPOption, OptionDescription, BoolOption, IntOption, StrOption
@@ -50,6 +53,7 @@ def test_requires_with_requires():
 
 def test_requires_invalid():
     a = BoolOption('activate_service', '', True)
+    a
     raises(ValueError, "IPOption('ip_address_service', '', requires='string')")
     raises(ValueError, "IPOption('ip_address_service', '', requires=[{'option': a, 'expected': False, 'action': 'disabled', 'unknown': True}])")
     raises(ValueError, "IPOption('ip_address_service', '', requires=[{'option': a, 'expected': False}])")
@@ -242,6 +246,30 @@ def test_requires_transitive():
     assert props == ['disabled']
 
 
+def test_requires_transitive_owner():
+    a = BoolOption('activate_service', '', True)
+    b = BoolOption('activate_service_web', '', True,
+                   requires=[{'option': a, 'expected': False, 'action': 'disabled'}])
+
+    d = IPOption('ip_address_service_web', '',
+                 requires=[{'option': b, 'expected': False, 'action': 'disabled'}])
+    od = OptionDescription('service', '', [a, b, d])
+    c = Config(od)
+    c.read_write()
+    c.activate_service
+    c.activate_service_web
+    c.ip_address_service_web
+    #no more default value
+    c.ip_address_service_web = '1.1.1.1'
+    c.activate_service = False
+    props = []
+    try:
+        c.ip_address_service_web
+    except PropertiesOptionError as err:
+        props = err.proptype
+    assert props == ['disabled']
+
+
 def test_requires_transitive_bis():
     a = BoolOption('activate_service', '', True)
     abis = BoolOption('activate_service_bis', '', True)
@@ -542,6 +570,7 @@ def test_requires_requirement_append():
 
 def test_requires_different_inverse():
     a = BoolOption('activate_service', '', True)
+    a
     raises(ValueError, "IPOption('ip_address_service', '', requires=[{'option': a, 'expected': True, 'action': 'disabled', 'inverse': True}, {'option': a, 'expected': True, 'action': 'disabled', 'inverse': False}])")
 
 
@@ -577,7 +606,65 @@ def test_set_item():
     raises(ValueError, 'c.cfgimpl_get_settings()[a] = ("test",)')
 
 
+def test_optiondescription_requires():
+    a = BoolOption('activate_service', '', True)
+    b = BoolOption('ip_address_service', '', multi=True)
+    a, b
+    OptionDescription('service', '', [b], requires=[{'option': a, 'expected': False, 'action': 'disabled'}])
+
+
+def test_optiondescription_requires_multi():
+    a = BoolOption('activate_service', '', True)
+    b = IPOption('ip_address_service', '', multi=True)
+    a, b
+    raises(ValueError, "OptionDescription('service', '', [a], requires=[{'option': b, 'expected': False, 'action': 'disabled'}])")
+
+
 def test_properties_conflict():
     a = BoolOption('activate_service', '', True)
+    a
     raises(ValueError, "IPOption('ip_address_service', '', properties=('disabled',), requires=[{'option': a, 'expected': False, 'action': 'disabled'}])")
     raises(ValueError, "od1 = OptionDescription('service', '', [a], properties=('disabled',), requires=[{'option': a, 'expected': False, 'action': 'disabled'}])")
+
+
+def test_master_slave_requires():
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True,
+                                   requires=[{'option': ip_admin_eth0, 'expected': '192.168.1.1', 'action': 'disabled'}])
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == []
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.2')
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None]
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.1.2']
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.1')
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[0] is None
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
+
+
+def test_master_slave_requires_no_master():
+    activate = BoolOption('activate', "Activer l'accès au réseau", True)
+    ip_admin_eth0 = StrOption('ip_admin_eth0', "ip réseau autorisé", multi=True)
+    netmask_admin_eth0 = StrOption('netmask_admin_eth0', "masque du sous-réseau", multi=True,
+                                   requires=[{'option': activate, 'expected': False, 'action': 'disabled'}])
+    interface1 = OptionDescription('ip_admin_eth0', '', [ip_admin_eth0, netmask_admin_eth0])
+    interface1.impl_set_group_type(groups.master)
+    maconfig = OptionDescription('toto', '', [activate, interface1])
+    cfg = Config(maconfig)
+    cfg.read_write()
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == []
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == []
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.2')
+    assert cfg.ip_admin_eth0.netmask_admin_eth0 == [None]
+    assert cfg.ip_admin_eth0.netmask_admin_eth0[0] is None
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.1.2']
+    cfg.activate = False
+    cfg.ip_admin_eth0.ip_admin_eth0.append('192.168.1.1')
+    assert cfg.ip_admin_eth0.ip_admin_eth0 == ['192.168.1.2', '192.168.1.1']
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0")
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[0]")
+    raises(PropertiesOptionError, "cfg.ip_admin_eth0.netmask_admin_eth0[1]")
