@@ -19,9 +19,9 @@ from time import time
 from copy import copy
 from logging import getLogger
 import weakref
-from tiramisu.error import (RequirementError, PropertiesOptionError,
-                            ConstError, ConfigError)
-from tiramisu.i18n import _
+from .error import (RequirementError, PropertiesOptionError,
+                    ConstError, ConfigError)
+from .i18n import _
 
 
 "Default encoding for display a Config if raise UnicodeEncodeError"
@@ -75,6 +75,9 @@ everything_frozen
     whole option in config are frozen (even if option have not frozen
     property)
 
+empty
+    raise mandatory PropertiesOptionError if multi or master have empty value
+
 validator
     launch validator set by user in option (this property has no effect
     for internal validator)
@@ -97,13 +100,19 @@ read_write
     you can set all variables not frozen
 """
 ro_append = set(['frozen', 'disabled', 'validator', 'everything_frozen',
-                'mandatory'])
+                'mandatory', 'empty'])
 ro_remove = set(['permissive', 'hidden'])
 rw_append = set(['frozen', 'disabled', 'validator', 'hidden'])
-rw_remove = set(['permissive', 'everything_frozen', 'mandatory'])
+rw_remove = set(['permissive', 'everything_frozen', 'mandatory', 'empty'])
+
+
+forbidden_set_properties = set(['force_store_value'])
 
 
 log = getLogger('tiramisu')
+#FIXME
+#import logging
+#logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)
 
 
 # ____________________________________________________________
@@ -114,11 +123,11 @@ class _NameSpace(object):
     """
 
     def __setattr__(self, name, value):
-        if name in self.__dict__:
+        if name in self.__dict__:  # pragma: optional cover
             raise ConstError(_("can't rebind {0}").format(name))
         self.__dict__[name] = value
 
-    def __delattr__(self, name):
+    def __delattr__(self, name):  # pragma: optional cover
         if name in self.__dict__:
             raise ConstError(_("can't unbind {0}").format(name))
         raise ValueError(name)
@@ -205,6 +214,7 @@ def populate_owners():
     """
     setattr(owners, 'default', owners.DefaultOwner('default'))
     setattr(owners, 'user', owners.Owner('user'))
+    setattr(owners, 'forced', owners.Owner('forced'))
 
     def addowner(name):
         """
@@ -213,39 +223,16 @@ def populate_owners():
         setattr(owners, name, owners.Owner(name))
     setattr(owners, 'addowner', addowner)
 
-
-def populate_multitypes():
-    """all multi option should have a type, this type is automaticly set do
-    not touch this
-
-    default
-        default's multi option set if not master or slave
-
-    master
-        master's option in a group with master's type, name of this option
-        should be the same name of the optiondescription
-
-    slave
-        slave's option in a group with master's type
-
-    """
-    setattr(multitypes, 'default', multitypes.DefaultMultiType('default'))
-    setattr(multitypes, 'master', multitypes.MasterMultiType('master'))
-    setattr(multitypes, 'slave', multitypes.SlaveMultiType('slave'))
-
-
 # ____________________________________________________________
-# populate groups, owners and multitypes with default attributes
+# populate groups and owners with default attributes
 groups = GroupModule()
 populate_groups()
 owners = OwnerModule()
 populate_owners()
-multitypes = MultiTypeModule()
-populate_multitypes()
 
 
 # ____________________________________________________________
-class Undefined():
+class Undefined(object):
     pass
 
 
@@ -269,13 +256,19 @@ class Property(object):
         :param propname: a predefined or user defined property name
         :type propname: string
         """
-        if self._opt is not None and self._opt._calc_properties is not None \
-                and propname in self._opt._calc_properties:
+        self._append(propname)
+
+    def _append(self, propname, save=True):
+        if self._opt is not None and self._opt.impl_getrequires() is not None \
+                and propname in self._opt.impl_get_calc_properties():  # pragma: optional cover
             raise ValueError(_('cannot append {0} property for option {1}: '
                                'this property is calculated').format(
-                                   propname, self._opt._name))
+                                   propname, self._opt.impl_getname()))
+        if propname in forbidden_set_properties:
+            raise ConfigError(_('cannot add those properties: {0}').format(propname))
         self._properties.add(propname)
-        self._setting._setproperties(self._properties, self._opt, self._path)
+        if save:
+            self._setting._setproperties(self._properties, self._path, force=True)
 
     def remove(self, propname):
         """Removes a property named propname
@@ -285,8 +278,7 @@ class Property(object):
         """
         if propname in self._properties:
             self._properties.remove(propname)
-            self._setting._setproperties(self._properties, self._opt,
-                                         self._path)
+            self._setting._setproperties(self._properties, self._path)
 
     def extend(self, propnames):
         """Extends properties to the existing properties
@@ -295,7 +287,8 @@ class Property(object):
         :type propnames: iterable of string
         """
         for propname in propnames:
-            self.append(propname)
+            self._append(propname, save=False)
+        self._setting._setproperties(self._properties, self._path)
 
     def reset(self):
         """resets the properties (does not **clear** the properties,
@@ -337,7 +330,7 @@ class Settings(object):
         old `SubConfig`, `Values`, `Multi` or `Settings`)
         """
         context = self.context()
-        if context is None:
+        if context is None:  # pragma: optional cover
             raise ConfigError(_('the context does not exist anymore'))
         return context
 
@@ -345,170 +338,180 @@ class Settings(object):
     # properties methods
     def __contains__(self, propname):
         "enables the pythonic 'in' syntaxic sugar"
-        return propname in self._getproperties(read_only=True)
+        return propname in self._getproperties(read_write=False)
 
     def __repr__(self):
-        return str(list(self._getproperties()))
+        return str(list(self._getproperties(read_write=False)))
 
     def __getitem__(self, opt):
-        path = self._get_path_by_opt(opt)
-        return self._getitem(opt, path)
+        path = opt.impl_getpath(self._getcontext())
+        return self.getproperties(opt, path)
 
-    def _getitem(self, opt, path):
-        return Property(self, self._getproperties(opt, path), opt, path)
+    def getproperties(self, opt, path, setting_properties=undefined):
+        return Property(self,
+                        self._getproperties(opt, path,
+                                            setting_properties=setting_properties),
+                        opt, path)
 
-    def __setitem__(self, opt, value):
-        raise ValueError('you should only append/remove properties')
+    def __setitem__(self, opt, value):  # pragma: optional cover
+        raise ValueError(_('you should only append/remove properties'))
 
     def reset(self, opt=None, _path=None, all_properties=False):
-        if all_properties and (_path or opt):
+        if all_properties and (_path or opt):  # pragma: optional cover
             raise ValueError(_('opt and all_properties must not be set '
                                'together in reset'))
         if all_properties:
             self._p_.reset_all_properties()
         else:
             if opt is not None and _path is None:
-                _path = self._get_path_by_opt(opt)
-            self._p_.reset_properties(_path)
+                _path = opt.impl_getpath(self._getcontext())
+            self._p_.delproperties(_path)
         self._getcontext().cfgimpl_reset_cache()
 
-    def _getproperties(self, opt=None, path=None, is_apply_req=True, read_only=False):
+    def _getproperties(self, opt=None, path=None,
+                       setting_properties=undefined, read_write=True,
+                       apply_requires=True, index=None):
+        """
+        """
         if opt is None:
             props = self._p_.getproperties(path, default_properties)
-            if not read_only:
-                props = copy(props)
         else:
-            if path is None:
+            if setting_properties is undefined:
+                setting_properties = self._getproperties(read_write=False)
+            if path is None:  # pragma: optional cover
                 raise ValueError(_('if opt is not None, path should not be'
                                    ' None in _getproperties'))
-            ntime = None
-            if is_apply_req and 'cache' in self and self._p_.hascache(path):
-                if 'expire' in self:
+            is_cached = False
+            if apply_requires:
+                if 'cache' in setting_properties and 'expire' in setting_properties:
                     ntime = int(time())
-                is_cached, props = self._p_.getcache(path, ntime)
-                if is_cached:
-                    if read_only:
-                        return props
-                    else:
-                        return copy(props)
-            props = self._p_.getproperties(path, opt._properties)
-            if not read_only:
-                props = copy(props)
-            if is_apply_req:
-                props |= self.apply_requires(opt, path)
-                if 'cache' in self:
-                    if 'expire' in self:
-                        if  ntime is None:
-                            ntime = int(time())
-                        ntime = ntime + expires_time
-                    self._p_.setcache(path, copy(props), ntime)
+                else:
+                    ntime = None
+                if 'cache' in setting_properties and self._p_.hascache(path, index):
+                    is_cached, props = self._p_.getcache(path, ntime, index)
+            if not is_cached:
+                props = self._p_.getproperties(path, opt.impl_getproperties())
+                if opt.impl_is_multi() and not opt.impl_is_master_slaves('slave'):
+                    props.add('empty')
+                if apply_requires:
+                    requires = self.apply_requires(opt, path, setting_properties, index)
+                    if requires != set([]):
+                        props = copy(props)
+                        props |= requires
+                    if 'cache' in setting_properties:
+                        if 'expire' in setting_properties:
+                            ntime = ntime + expires_time
+                        self._p_.setcache(path, props, ntime, index)
+        if read_write:
+            props = copy(props)
         return props
 
     def append(self, propname):
         "puts property propname in the Config's properties attribute"
         props = self._p_.getproperties(None, default_properties)
-        props.add(propname)
-        self._setproperties(props, None, None)
+        if propname not in props:
+            props.add(propname)
+            self._setproperties(props, None)
 
     def remove(self, propname):
         "deletes property propname in the Config's properties attribute"
         props = self._p_.getproperties(None, default_properties)
         if propname in props:
             props.remove(propname)
-            self._setproperties(props, None, None)
+            self._setproperties(props, None)
 
     def extend(self, propnames):
         for propname in propnames:
             self.append(propname)
 
-    def _setproperties(self, properties, opt, path):
-        """save properties for specified opt
+    def _setproperties(self, properties, path, force=False):
+        """save properties for specified path
         (never save properties if same has option properties)
         """
-        if opt is None:
-            self._p_.setproperties(None, properties)
-        else:
-            if opt._calc_properties is not None:
-                properties -= opt._calc_properties
-            if set(opt._properties) == properties:
-                self._p_.reset_properties(path)
-            else:
-                self._p_.setproperties(path, properties)
+        if not force:
+            forbidden_properties = forbidden_set_properties & properties
+            if forbidden_properties:
+                raise ConfigError(_('cannot add those properties: {0}').format(
+                    ' '.join(forbidden_properties)))
+        self._p_.setproperties(path, properties)
         self._getcontext().cfgimpl_reset_cache()
 
     #____________________________________________________________
-    def validate_properties(self, opt_or_descr, is_descr, is_write, path,
-                            value=undefined, force_permissive=False,
-                            force_properties=None, force_permissives=None):
+    def validate_properties(self, opt_or_descr, is_descr, check_frozen, path,
+                            value=None, force_permissive=False,
+                            setting_properties=undefined,
+                            self_properties=undefined,
+                            index=None):
         """
         validation upon the properties related to `opt_or_descr`
 
         :param opt_or_descr: an option or an option description object
         :param force_permissive: behaves as if the permissive property
                                  was present
-        :param force_properties: set() with properties that is force to add
-                                 in global properties
-        :param force_permissives: set() with permissives that is force to add
-                                 in global permissives
         :param is_descr: we have to know if we are in an option description,
                          just because the mandatory property
                          doesn't exist here
 
-        :param is_write: in the validation process, an option is to be modified,
+        :param check_frozen: in the validation process, an option is to be modified,
                          the behavior can be different
                          (typically with the `frozen` property)
         """
         # opt properties
-        properties = self._getproperties(opt_or_descr, path)
-        self_properties = self._getproperties()
+        if setting_properties is undefined:
+            setting_properties = self._getproperties(read_write=False)
+        if self_properties is not undefined:
+            properties = copy(self_properties)
+        else:
+            properties = self._getproperties(opt_or_descr, path,
+                                             setting_properties=setting_properties,
+                                             index=index)
         # remove opt permissive
         # permissive affect option's permission with or without permissive
         # global property
         properties -= self._p_.getpermissive(path)
         # remove global permissive if need
-        if force_permissive is True or 'permissive' in self_properties:
+        if force_permissive is True or 'permissive' in setting_properties:
             properties -= self._p_.getpermissive()
-        if force_permissives is not None:
-            properties -= force_permissives
-
-        # global properties
-        if force_properties is not None:
-            self_properties.update(force_properties)
 
         # calc properties
-        properties &= self_properties
-        # mandatory and frozen are special properties
-        if is_descr:
-            properties -= frozenset(('mandatory', 'frozen'))
-        else:
-            if value is not undefined and 'mandatory' in properties and \
+        properties &= setting_properties
+        if not is_descr:
+            #mandatory
+            if 'mandatory' in properties and \
                     not self._getcontext().cfgimpl_get_values()._isempty(
-                        opt_or_descr, value):
+                        opt_or_descr, value, index=index):
                 properties.remove('mandatory')
-            if is_write and 'everything_frozen' in self_properties:
+            elif 'empty' in properties and \
+                    'empty' in setting_properties and \
+                    self._getcontext().cfgimpl_get_values()._isempty(
+                        opt_or_descr, value, force_allow_empty_list=True):
+                properties.add('mandatory')
+            # should return 'frozen' only when tried to modify a value
+            if check_frozen and 'everything_frozen' in setting_properties:
                 properties.add('frozen')
-            elif 'frozen' in properties and not is_write:
+            elif 'frozen' in properties and not check_frozen:
                 properties.remove('frozen')
+            if 'empty' in properties:
+                properties.remove('empty')
         # at this point an option should not remain in properties
         if properties != frozenset():
             props = list(properties)
             if 'frozen' in properties:
-                raise PropertiesOptionError(_('cannot change the value for '
-                                              'option {0} this option is'
-                                              ' frozen').format(
-                                                  opt_or_descr._name),
-                                            props)
+                return PropertiesOptionError(_('cannot change the value for '
+                                               'option {0} this option is'
+                                               ' frozen').format(
+                                                   opt_or_descr.impl_getname()),
+                                             props)
             else:
-                if opt_or_descr._opt_type == 'optiondescription':
+                if is_descr:
                     opt_type = 'optiondescription'
                 else:
                     opt_type = 'option'
-
-                raise PropertiesOptionError(_("trying to access to an {0} "
-                                              "named: {1} with properties {2}"
-                                              "").format(opt_type,
-                                                         opt_or_descr._name,
-                                                         str(props)), props)
+                return PropertiesOptionError(_("trying to access to an {0} "
+                                               "named: {1} with properties {2}"
+                                               "").format(opt_type,
+                                                          opt_or_descr._name,
+                                                          str(props)), props)
 
     def setpermissive(self, permissive, opt=None, path=None):
         """
@@ -521,15 +524,15 @@ class Settings(object):
                     instead of passing a :class:`tiramisu.option.Option()` object.
         """
         if opt is not None and path is None:
-            path = self._get_path_by_opt(opt)
-        if not isinstance(permissive, tuple):
+            path = opt.impl_getpath(self._getcontext())
+        if not isinstance(permissive, tuple):  # pragma: optional cover
             raise TypeError(_('permissive must be a tuple'))
         self._p_.setpermissive(path, permissive)
 
     #____________________________________________________________
     def setowner(self, owner):
         ":param owner: sets the default value for owner at the Config level"
-        if not isinstance(owner, owners.Owner):
+        if not isinstance(owner, owners.Owner):  # pragma: optional cover
             raise TypeError(_("invalid generic owner {0}").format(str(owner)))
         self._owner = owner
 
@@ -538,10 +541,16 @@ class Settings(object):
 
     #____________________________________________________________
     def _read(self, remove, append):
-        for prop in remove:
-            self.remove(prop)
-        for prop in append:
-            self.append(prop)
+        props = self._p_.getproperties(None, default_properties)
+        modified = False
+        if remove & props != set([]):
+            props = props - remove
+            modified = True
+        if append & props != append:
+            props = props | append
+            modified = True
+        if modified:
+            self._setproperties(props, None)
 
     def read_only(self):
         "convenience method to freeze, hide and disable"
@@ -562,7 +571,7 @@ class Settings(object):
         else:
             self._p_.reset_all_cache()
 
-    def apply_requires(self, opt, path):
+    def apply_requires(self, opt, path, setting_properties, index):
         """carries out the jit (just in time) requirements between options
 
         a requirement is a tuple of this form that comes from the option's
@@ -606,38 +615,46 @@ class Settings(object):
         :param path: the option's path in the config
         :type path: str
         """
-        if opt._requires is None:
+        if opt.impl_getrequires() == []:
             return frozenset()
 
         # filters the callbacks
         calc_properties = set()
         context = self._getcontext()
-        for requires in opt._requires:
+        for requires in opt.impl_getrequires():
             for require in requires:
                 option, expected, action, inverse, \
                     transitive, same_action = require
-                reqpath = self._get_path_by_opt(option)
-                if reqpath == path or reqpath.startswith(path + '.'):
+                reqpath = option.impl_getpath(context)
+                if reqpath == path or reqpath.startswith(path + '.'):  # pragma: optional cover
                     raise RequirementError(_("malformed requirements "
                                              "imbrication detected for option:"
                                              " '{0}' with requirement on: "
                                              "'{1}'").format(path, reqpath))
-                try:
-                    value = context.getattr(reqpath, force_permissive=True)
-                except PropertiesOptionError as err:
-                    if not transitive:
-                        continue
-                    properties = err.proptype
-                    if same_action and action not in properties:
-                        raise RequirementError(_("option '{0}' has "
-                                                 "requirement's property "
-                                                 "error: "
-                                                 "{1} {2}").format(opt._name,
-                                                                   reqpath,
-                                                                   properties))
-                    # transitive action, force expected
-                    value = expected[0]
-                    inverse = False
+                if option.impl_is_multi():
+                    idx = index
+                else:
+                    idx = None
+                value = context.getattr(reqpath, force_permissive=True,
+                                        _setting_properties=setting_properties,
+                                        index=idx, returns_raise=True)
+                if isinstance(value, Exception):
+                    if isinstance(value, PropertiesOptionError):
+                        if not transitive:
+                            continue
+                        properties = value.proptype
+                        if same_action and action not in properties:  # pragma: optional cover
+                            raise RequirementError(_("option '{0}' has "
+                                                     "requirement's property "
+                                                     "error: "
+                                                     "{1} {2}").format(opt._name,
+                                                                       reqpath,
+                                                                       properties))
+                        # transitive action, force expected
+                        value = expected[0]
+                        inverse = False
+                    else:
+                        raise value
                 if (not inverse and
                         value in expected or
                         inverse and value not in expected):
@@ -646,14 +663,6 @@ class Settings(object):
                     break
         return calc_properties
 
-    def _get_path_by_opt(self, opt):
-        """just a wrapper to get path in optiondescription's cache
-
-        :param opt: `Option`'s object
-        :returns: path
-        """
-        return self._getcontext().cfgimpl_get_description().impl_get_path_by_opt(opt)
-
     def get_modified_properties(self):
         return self._p_.get_modified_properties()
 
