@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2013 Team tiramisu (see AUTHORS for all contributors)
+# Copyright (C) 2013-2014 Team tiramisu (see AUTHORS for all contributors)
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -25,5 +25,7 @@ use it. But if something goes wrong, you will lost your modifications.
 from .value import Values
 from .setting import Settings
 from .storage import setting, Storage, list_sessions, delete_session
+from .option import StorageBase, StorageOptionDescription
 
-__all__ = (setting, Values, Settings, Storage, list_sessions, delete_session)
+__all__ = (setting, Values, Settings, Storage, list_sessions, delete_session,
+           StorageBase, StorageOptionDescription)
